@@ -1,3 +1,20 @@
+// Copyright (C) 2014 - 2015  Jack O. Wasey
+//
+// This file is part of icd9.
+//
+// icd9 is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// icd9 is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with icd9. If not, see <http://www.gnu.org/licenses/>.
+
 // [[Rcpp::interfaces(r, cpp)]]
 #include <Rcpp.h>
 #include <is.h>
@@ -10,7 +27,7 @@ using namespace Rcpp;
 // [[Rcpp::export]]
 CharacterVector icd9MajMinToCodeShim(const CharacterVector mjr,
 		const CharacterVector mnr, bool isShort) {
-#ifdef ICD9_DEBUG
+#ifdef ICD9_DEBUG_TRACE
 	Rcout << "icd9MajMinToCode\n";
 	if (mjr.size() != mnr.size())
 	Rcpp::stop("aborting because mjr and mnr lengths differ");
@@ -20,7 +37,6 @@ CharacterVector icd9MajMinToCodeShim(const CharacterVector mjr,
 	CharacterVector::const_iterator n = mnr.begin();
 
 	for (; j != mjr.end() && n != mnr.end(); ++j, ++n) {
-		//std::string mjrelem = as<std::string>(*j);
 		String mjrelem = *j;
 		if (mjrelem == NA_STRING) {
 			out.push_back(NA_STRING);
@@ -61,7 +77,6 @@ CharacterVector icd9MajMinToCodeShim(const CharacterVector mjr,
 		out.push_back(smj);
 
 	}
-	// String chr_s = NA_STRING; // TODO
 
 	return wrap(out);
 }
@@ -69,7 +84,7 @@ CharacterVector icd9MajMinToCodeShim(const CharacterVector mjr,
 // [[Rcpp::export]]
 CharacterVector icd9MajMinToShortShim(const CharacterVector mjr,
 		const CharacterVector mnr) {
-#ifdef ICD9_DEBUG
+#ifdef ICD9_DEBUG_TRACE
 	Rcout << "icd9MajMinToShort: mjr.size() = " << mjr.size()
 	<< " and mnr.size() = " << mnr.size() << "\n";
 #endif
@@ -78,7 +93,7 @@ CharacterVector icd9MajMinToShortShim(const CharacterVector mjr,
 		Rcpp::stop("icd9MajMinToShort, length of mjrs and mnrs must be equal.");
 	}
 	if (mjr.size() == 1) {
-#ifdef ICD9_DEBUG
+#ifdef ICD9_DEBUG_TRACE
 		Rcout << "icd9MajMinToShort: mjr.size() = 1\n";
 #endif
 		CharacterVector newmjr(mnr.size(), mjr[0]);
@@ -94,14 +109,14 @@ CharacterVector icd9MajMinToDecimalShim(const CharacterVector mjr,
 }
 
 //' @rdname convert
-//' @export
+//' @keywords internal manip
 // [[Rcpp::export]]
 CharacterVector icd9PartsToShort(const List parts) {
 	return icd9MajMinToCodeShim(parts["major"], parts["minor"], true);
 }
 
 //' @rdname convert
-//' @export
+//' @keywords internal manip
 // [[Rcpp::export]]
 CharacterVector icd9PartsToDecimal(const List parts) {
 	return icd9MajMinToCodeShim(parts["major"], parts["minor"], false);
@@ -112,10 +127,11 @@ List icd9MajMinToPartsShim(const CharacterVector mjr,
 		const CharacterVector mnr) {
 	List returned_frame = List::create(_["major"] = mjr, _["minor"] = mnr);
 
-	// TODO: can do this with lists, ?no need for a data frame
 	StringVector sample_row = returned_frame(0);
 	IntegerVector row_names = seq_along(sample_row);
 	returned_frame.attr("row.names") = row_names;
+  // doesn't actually need a data frame, although it is barely distinguishable
+  // from a list, and not costly to construct in this manner.
 	returned_frame.attr("class") = "data.frame";
 
 	return returned_frame;
@@ -129,10 +145,9 @@ List icd9MajMinToParts_listShim(const CharacterVector mjr,
 }
 
 //' @rdname convert
-//' @export
+//' @keywords internal manip
 // [[Rcpp::export]]
-List icd9ShortToParts(const CharacterVector icd9Short, const String minorEmpty =
-		"") {
+List icd9ShortToParts(const CharacterVector icd9Short, const String minorEmpty = "") {
 
 	CharacterVector mjr(icd9Short.size());
 	CharacterVector mnr(icd9Short.size());
@@ -144,53 +159,53 @@ List icd9ShortToParts(const CharacterVector icd9Short, const String minorEmpty =
 		}
 
 		std::string s = as<std::string>(icd9Short[i]);
-		s = strimCpp(s); // do i need to convert?
+		s = strimCpp(s);
+		std::string::size_type sz = s.size();
 
-		if (!icd9IsASingleE(s.c_str())) { // not an E code
-			switch (s.size()) {
+		if (icd9IsASingleE(s.c_str())) { // E code
+		  switch (sz) {
+		  case 2:
+		  case 3:
+		  case 4:
+		    mjr[i] = s.substr(0, sz);
+		    mnr[i] = minorEmpty;
+		    break;
+		  case 5:
+		    mjr[i] = s.substr(0, 4);
+		    mnr[i] = s.substr(4, 1);
+		    break;
+		  default:
+		    mjr[i] = NA_STRING;
+		  mnr[i] = NA_STRING;
+		  continue;
+		  }
+		} else { // not an E code
+			switch (sz) {
 			case 1:
 			case 2:
 			case 3:
-				mjr[i] = s.substr(0, s.size());
+				mjr[i] = s.substr(0, sz);
 				mnr[minorEmpty];
 				continue;
 			case 4:
 			case 5:
 				mjr[i] = s.substr(0, 3);
-				mnr[i] = s.substr(3, s.size() - 3);
+				mnr[i] = s.substr(3, sz - 3);
 				continue;
 			default:
 				mjr[i] = NA_STRING;
 				mnr[i] = NA_STRING;
 				continue;
 			}
-		} else { // E code
+		}
 
-			switch (s.size()) {
-			case 2:
-			case 3:
-			case 4:
-				mjr[i] = s.substr(0, s.size());
-				mnr[i] = minorEmpty;
-				break;
-			case 5:
-				mjr[i] = s.substr(0, 4);
-				mnr[i] = s.substr(4, 1);
-				break;
-			default:
-				mjr[i] = NA_STRING;
-				mnr[i] = NA_STRING;
-				continue;
-			}
-		} // E code
-		  //mjr[i] = icd9AddLeadingZeroesmjrSingle(mjr[i]); // or loop through them all again...
 	} // for
 
 	return icd9MajMinToPartsShim(icd9AddLeadingZeroesMajorShim(mjr), mnr);
 }
 
 //' @rdname convert
-//' @export
+//' @keywords internal manip
 // [[Rcpp::export]]
 List icd9DecimalToParts(const CharacterVector icd9Decimal,
 		const String minorEmpty = "") {
@@ -211,8 +226,12 @@ List icd9DecimalToParts(const CharacterVector icd9Decimal,
 			mnrs.push_back(NA_STRING);
 			continue;
 		}
-		std::string thiscode = as<std::string>(*it); // Rcpp::String doesn't implement many functions.
-		thiscode = strimCpp(thiscode); // TODO: update in place.
+		// Rcpp::String doesn't implement many functions, so using STL. A FAST way
+		// would be to use String's function get_cstring, and recode the trim
+		// functions to take const char *. This would avoid the type change AND be
+		// faster trimming.
+		std::string thiscode = as<std::string>(*it);
+		thiscode = strimCpp(thiscode); // This updates 'thisccode' by reference, no copy
 		std::size_t pos = thiscode.find(".");
 		// substring parts
 		std::string mjrin;
@@ -230,21 +249,25 @@ List icd9DecimalToParts(const CharacterVector icd9Decimal,
 	return List::create(_["major"] = mjrs, _["minor"] = mnrs);
 }
 
-//' @rdname convert
+//' @title Convert ICD-9 codes between short and decimal forms
+//' @template icd9-short
+//' @template icd9-decimal
 //' @export
 // [[Rcpp::export]]
 CharacterVector icd9ShortToDecimal(const CharacterVector icd9Short) {
 	return icd9PartsToDecimal(icd9ShortToParts(icd9Short));
 }
 
-//' @rdname convert
+//' @rdname icd9ShortToDecimal
 //' @export
 // [[Rcpp::export]]
 CharacterVector icd9DecimalToShort(const CharacterVector icd9Decimal) {
 	return icd9PartsToShort(icd9DecimalToParts(icd9Decimal));
 }
 
-//' @rdname convert
+//' @title Get major (three-digit) part of ICD-9 codes
+//' @template icd9-any
+//' @template isShort
 //' @export
 // [[Rcpp::export]]
 CharacterVector icd9GetMajor(const CharacterVector icd9, const bool isShort) {
