@@ -1,3 +1,20 @@
+// Copyright (C) 2014 - 2015  Jack O. Wasey
+//
+// This file is part of icd9.
+//
+// icd9 is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// icd9 is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with icd9. If not, see <http://www.gnu.org/licenses/>.
+
 // [[Rcpp::interfaces(r, cpp)]]
 #include <Rcpp.h>
 #include <ranges.h>
@@ -90,7 +107,7 @@ CharacterVector icd9ExpandMinorShim(std::string mnr, bool isE = false) {
 			return CharacterVector::create();
 		}
 	} else {
-		// is E code, so minor is just one character
+		// is E code, so minor must be just one character
 		switch (mnr.size()) {
 		case 0:
 			return CharacterVector::create("", "0", "1", "2", "3", "4", "5",
@@ -98,7 +115,7 @@ CharacterVector icd9ExpandMinorShim(std::string mnr, bool isE = false) {
 		case 1:
 			return mnr;
 		default:
-			Rcpp::stop("too many characters in E code minor part\n");
+			Rcpp::stop("too many characters for an E code minor part\n");
 		}
 	}
 	return (NA_STRING); // should never get here
@@ -165,8 +182,7 @@ CharacterVector icd9ChildrenCpp(CharacterVector icd9, bool isShort,
 //' @template icd9-any
 //' @template isShort
 //' @param isShortReference logical, see argument \code{isShort}
-//' @return logical vector of which icd9 match or are subcategory of
-//'   \code{icd9Reference}
+//' @return logical vector
 //' @keywords internal
 // [[Rcpp::export]]
 LogicalVector icd9InReferenceCode(CharacterVector icd9,
@@ -180,7 +196,6 @@ LogicalVector icd9InReferenceCode(CharacterVector icd9,
 	CharacterVector y = icd9ChildrenCpp(icd9Reference, isShortReference, false);
 	if (!isShortReference)
 		y = icd9DecimalToShort(y);
-	// Rcpp match is not quite as good as R:
-	LogicalVector res = !is_na(match(x, y));
+   LogicalVector res = !is_na(match(x, y));
 	return res;
 }
