@@ -1,3 +1,20 @@
+# Copyright (C) 2014 - 2015  Jack O. Wasey
+#
+# This file is part of icd9.
+#
+# icd9 is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# icd9 is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with icd9. If not, see <http:#www.gnu.org/licenses/>.
+
 context("icd9 type conversions")
 
 test_that("extract decimal parts - invalid or empty input", {
@@ -13,7 +30,6 @@ test_that("extract decimal parts - invalid or empty input", {
 })
 
 test_that("extract decimal parts - valid inputs", {
-  # zero is technically "valid", means no code. TODO: apply elsewhere?
   expect_equal(icd9DecimalToParts("0"), list(major = "000", minor = ""))
   expect_equal(icd9DecimalToParts("000"), list(major = "000", minor = ""))
   expect_equal(icd9DecimalToParts("V1.2"), list(major = "V01", minor = "2"))
@@ -64,10 +80,13 @@ test_that("extract decimal parts - valid inputs", {
 
 test_that("icd9 decimal to short form, bad codes", {
   expect_equal(icd9DecimalToShort(character()), character())
-  skip("TODO: flesh out")
+  expect_equal(icd9DecimalToShort("harissa"), NA_character_)
+  expect_equal(icd9DecimalToShort("12345.678"), NA_character_)
+  expect_equal(icd9DecimalToShort("123 67"), NA_character_)
+  expect_true(all(is.na(icd9DecimalToShort(c("07022", "07023")))))
 })
-test_that("icd9 decimal to short form", {
 
+test_that("icd9 decimal to short form", {
   expect_equal(icd9DecimalToShort("1"), "001")
   expect_equal(icd9DecimalToShort("1.1"), "0011")
   expect_equal(icd9DecimalToShort("1.23"), "00123")
@@ -85,8 +104,6 @@ test_that("icd9 decimal to short form", {
   expect_equal(icd9DecimalToShort(c("1", "", "991.23")),
                c("001", NA_character_, "99123"))
 
-  expect_error(icd9DecimalToShort(c("07022","07023"), validate = TRUE))
-
 })
 
 test_that("short to decimal, numbers", {
@@ -95,21 +112,15 @@ test_that("short to decimal, numbers", {
   expect_equal(icd9DecimalToShort("1"), "001")
   expect_equal(icd9DecimalToShort("22"), "022")
   expect_equal(icd9DecimalToShort("345"), "345")
-})
-
-test_that("short to decimal with flags", {
-  #TODO more permutations to expand here:
   expect_equal(icd9ShortToDecimal("013"), "013")
   expect_equal(icd9ShortToDecimal("V013"), "V01.3")
 })
 
 test_that("short to decimal bad input", {
-
   expect_equal(icd9ShortToDecimal(character()), character())
   expect_equal(icd9ShortToDecimal("valsalva"), NA_character_)
   expect_equal(icd9ShortToDecimal("123456"), NA_character_)
   expect_equal(icd9ShortToDecimal(""), NA_character_)
-  #expect_equal(icd9ShortToDecimal("-1"), NA_character_)
   # NA is not character type, so expect error.
   expect_equal(icd9ShortToDecimal(NA), NA_character_)
   # NA is not character type, so expect error.
@@ -174,12 +185,12 @@ test_that("running short to decimal conversion before and after expansion
           })
 
 test_that("parts to decimal", {
-  expect_that(icd9PartsToDecimal(data.frame(major = "100", minor = NA)), equals("100"))
-  expect_that(icd9PartsToDecimal(data.frame(major = "100", minor = "")), equals("100"))
-  expect_that(icd9PartsToDecimal(data.frame(major = "100", minor = "1")), equals("100.1"))
-  expect_that(icd9MajMinToDecimal("100", NA), equals("100"))
-  expect_that(icd9MajMinToDecimal("100", ""), equals("100"))
-  expect_that(icd9MajMinToDecimal("100", "1"), equals("100.1"))
+  expect_that(icd9PartsToDecimal(data.frame(major = "100", minor = NA)), testthat::equals("100"))
+  expect_that(icd9PartsToDecimal(data.frame(major = "100", minor = "")), testthat::equals("100"))
+  expect_that(icd9PartsToDecimal(data.frame(major = "100", minor = "1")), testthat::equals("100.1"))
+  expect_that(icd9MajMinToDecimal("100", NA), testthat::equals("100"))
+  expect_that(icd9MajMinToDecimal("100", ""), testthat::equals("100"))
+  expect_that(icd9MajMinToDecimal("100", "1"), testthat::equals("100.1"))
 })
 
 test_that("parts to short invalid inputs", {
@@ -188,22 +199,10 @@ test_that("parts to short invalid inputs", {
 
   expect_equal(icd9PartsToShort(dfempty), character())
   expect_equal(icd9PartsToShort(dfe2), NA_character_)
-  # TODO: convert to get NAs back instead of errors
-  #   expect_error(icd9PartsToShort(list(major = "turbottt", minor = "23")))
-  #   expect_error(icd9PartsToShort(list(major = "", minor = "23")))
-  #   expect_error(icd9PartsToShort(list(major = "turbottt", minor = "")))
-  #   expect_error(icd9PartsToShort(list(major = "", minor = "")))
-  #   expect_error(icd9PartsToShort(list(major = "turbottt", minor = NA)))
-  #   expect_error(icd9PartsToShort(list(major = "", minor = NA)))
-
-  expect_equal(icd9PartsToShort(list(major = NA, minor = "trout")),
-               NA_character_)
-  expect_equal(icd9PartsToShort(list(major = NA, minor = "23")),
-               NA_character_)
-  expect_equal(icd9PartsToShort(list(major = NA, minor = "")),
-               NA_character_)
-  expect_equal(icd9PartsToShort(list(major = NA, minor = NA)),
-               NA_character_)
+  expect_equal(icd9PartsToShort(list(major = NA, minor = "trout")), NA_character_)
+  expect_equal(icd9PartsToShort(list(major = NA, minor = "23")), NA_character_)
+  expect_equal(icd9PartsToShort(list(major = NA, minor = "")), NA_character_)
+  expect_equal(icd9PartsToShort(list(major = NA, minor = NA)), NA_character_)
 
 })
 
@@ -249,14 +248,13 @@ test_that("parts to short V code inputs", {
 
 test_that("icd9 parts to short: don't allow cycling.", {
   expect_error(icd9MajMinToShort(c("123", "34", "56"), c("1", "20")))
-  expect_error(icd9MajMinToShort(c("123", "34"), c("1", "20", "45"))) # causes hang only when compiled with MinGW GCC 4.9 in Rtools 3.2 on 64 bit
+  # causes hang only when compiled with MinGW GCC 4.9 in Rtools 3.2 on 64 bit
+  expect_error(icd9MajMinToShort(c("123", "34"), c("1", "20", "45")))
 })
 
 test_that("Windows Rtools 3.2 hang test", {
-  skip("don't crash winbuilder 64 bit R-devel 3.2 with Rtools 3.3 (GCC 4.9)")
   expect_error(icd9MajMinToShort(c("123", "34"), c("1", "20", "45")))
-  # see Rcpp issue #276. Now updated to use Rf_error instead of Rcpp::stop
-
+  # see Rcpp issue #276.
 })
 
 test_that("icd9 parts to short form V and E input, mismatched lengths", {
@@ -265,8 +263,8 @@ test_that("icd9 parts to short form V and E input, mismatched lengths", {
   expect_equal(icd9MajMinToShort("V01", c("0", "1")), c("V010", "V011"))
 })
 
-test_that("convert list of icd-9 ranges (e.g. chapter defintions to comorbidity map", {
-
+test_that("convert list of icd-9 ranges (e.g. chapter defintions to comorbidity map)", {
+  skip_on_cran()
   ooe <- data.frame(visitId = sprintf("pt%02d", seq_along(one_of_each)), icd9 = one_of_each)
 
   test.map <- icd9ChaptersToMap(icd9::icd9Chapters)
